import javax.swing.*;
import java.awt.Component;

/**
 * Clase que implementa la botonera de la cabina de un ascensor en el simulador.
 * Permite manejar la funcionalidad de los distintos botones de piso e indicar mediante estos el piso objetivo.
 *
 * @version 0.4
 */
public class BotoneraCabina extends Botonera
{
    //#################################
	private Boton[] botones;
	private BotoneraCabinaView view;

    //#################################
    /**
     * Constructor publico.
     * Inicializa el objeto y crea la vista del panel que se despliega en pantalla.
     * @see BotoneraCabinaView
     * @see ControlUnit
     * @param n_pisos   Numero de pisos a implementar.
     * @param cu        Unidad de control.
     * @param location  Indica donde se encuentra la botonera, en el caso de la cabina location = 0.
     */
	public BotoneraCabina(int n_pisos, ControlUnit cu, int location)
	{
		super(cu, location);
		view 	= new BotoneraCabinaView(1);
		botones = new Boton[n_pisos];
		for(int i = n_pisos; i > 0; i--)
		{
			botones[i-1] = new Boton(new ImageIcon(String.format("%d_Off.png",i)), new ImageIcon(String.format("%d_On.png",i)), this);
			view.addButton(botones[i-1]);
			botones[i-1].setNewDimension(50);
			botones[i-1].getView().setAlignmentX(Component.CENTER_ALIGNMENT);
		}
	}

    //#################################
    /**
     * Permite quitar la solicitud de ascensor de un piso determinado.
     * @param i         Numero de piso.
     */
	public void resetFloorRequest(int i)
	{
		botones[i-1].turnOFF();
	}
    /**
     * Retorna un booleano que indica si el piso ha solicitado el ascensor o no.
     * @param i         Numero de piso.
     * @return          Si retorna "true" el piso i fue solicitado desde la cabina, "false" en caso contrario.
     */
	public boolean isFloorRequested(int i)
	{
		return botones[i-1].getState();
	}
    /**
     * Retorna el numero del primer piso solicitado desde la cabina.
     * @return          Numero del piso solicitado.
     */
	public int searchBtnON()
	{
		for(int i = 0; i < botones.length; i++)
			if (botones[i].getState())
				return i+1;
		return 0;
	}
    /**
     * Retorna un string con el estado de todos los botones de la cabina.
     * @return          Lista de estado de botones, "1" si ha solicitado piso o "0" en caso contrario.
     */
	public String toString()
	{
		String s="";
		for(Boton b: botones)
			s+=b.toString();
		return s;
	}
    //******************************************
    //***** METODOS OBLIGADOS POR HERENCIA *****
    //******************************************
    /**
     * Permite indicar en la cabina en que piso se encuentra el ascensor.
     * @param cabinFloor    Numero de piso.
     */
    public void setFloorIndicator(int cabinFloor)
    {
        view.setFloorIndicator(cabinFloor);
    }
    /**
     * Permite definir el tamaño de los botones para ser presentados en la simulacion.
     * @param xy        Tamaño del boton en pixeles. En esta implementacion sera: Largo = Ancho.
     */
    public void setBotonNewDimension(int xy)		// In case the buttons need size adjustments
    {
        for (Boton b:botones)
            b.setNewDimension(xy);
    }
    /**
     * Retorna un elemento JPanel correspondiente a la botonera de la cabina y el indicador de piso.
     * @return          JPanel.
     */
    public JPanel getView()
    {
        return view.getView();
    }
}